Exception: Signoff::DefinitionError

Inherits:
Error
  • Object
show all
Defined in:
lib/signoff/errors.rb

Overview

Raised while building a workflow definition with the DSL when the definition is invalid (duplicate states, unknown transition targets, missing initial state, …).