Exception: Signoff::InvalidTransitionError

Inherits:
Error
  • Object
show all
Defined in:
lib/signoff/errors.rb

Overview

Raised at runtime when a transition is not permitted by the workflow definition (no edge from the current state, ambiguous target, rejecting a finalized record, …).